# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/b_utils.ipynb.

# %% auto 0
__all__ = ['print_progress']

# %% ../nbs/b_utils.ipynb 2
def print_progress(iteration, total, prefix='', length=30, fill='█'):
    percent = ("{0:.1f}").format(100 * (iteration / float(total)))
    filled_length = int(length * iteration // total)
    bar = fill * filled_length + '-' * (length - filled_length)
    return f'\r{prefix} |{bar}| {percent}%'


